create or replace function pgq.batch_event_sql(x_batch_id bigint)
returns text as $$
--
-- return query that should return events for this tick
--
declare
    rec           record;
    sql           text;
    tbl           text;
    arr           text;
    part          text;
    select_fields text;
    retry_expr    text;
    batch         record;
begin
    select b.tick_id, b.prev_tick_id, b.consumer_id, b.queue_id,
           get_snapshot_xmax(prev.snapshot) as tx_start,
           get_snapshot_xmax(cur.snapshot) as tx_end,
           prev.snapshot as prev_snapshot,
           cur.snapshot as cur_snapshot
        into batch
        from pgq.active_batch b, pgq.tick prev, pgq.tick cur
        where b.batch_id = x_batch_id
          and prev.queue_id = b.queue_id
          and prev.tick_id = b.prev_tick_id
          and cur.queue_id = b.queue_id
          and cur.tick_id = b.tick_id;
    if not found then
        raise exception 'batch not found';
    end if;

    -- load older transactions
    arr := '';
    for rec in
        -- active tx-es in prev_snapshot that were committed in cur_snapshot
        select id1 from
            get_snapshot_active(batch.prev_snapshot) id1 left join
            get_snapshot_active(batch.cur_snapshot) id2 on (id1 = id2)
        where id2 is null
        order by 1 desc
    loop
        -- try to avoid big IN expression, so try to include nearby
        -- tx'es into range
        if batch.tx_start - 100 <= rec.id1 then
            batch.tx_start := rec.id1;
        else
            if arr = '' then
                arr := rec.id1;
            else
                arr := arr || ',' || rec.id1;
            end if;
        end if;
    end loop;

    select_fields := 'select ev.event_id, ev.ref_id, ev.payload, ev.producer,'
        || ' ev.creation_date, ev.event_txid, ev.retry_count';
    retry_expr :=  ' and (consumer_id is null or consumer_id = '
        || batch.consumer_id || ')';

    -- now generate query that goes over all potential tables
    sql := '';
    for rec in
        select xtbl from pgq.batch_event_tables(x_batch_id) xtbl
    loop
        tbl := rec.xtbl;
        -- this gets newer queries that definitely are not in prev_snapshot
        part := select_fields
            || ' from pgq.tick cur, pgq.tick prev, ' || tbl || ' ev '
            || ' where cur.tick_id = ' || batch.tick_id
            || ' and cur.queue_id = ' || batch.queue_id
            || ' and prev.tick_id = ' || batch.prev_tick_id
            || ' and prev.queue_id = ' || batch.queue_id
            || ' and ev.event_txid >= ' || batch.tx_start
            || ' and ev.event_txid <= ' || batch.tx_end
            || ' and txid_in_snapshot(ev.event_txid, cur.snapshot)'
            || ' and not txid_in_snapshot(ev.event_txid, prev.snapshot)'
            || retry_expr;
        -- now include older tx-es, that were ongoing
        -- at the time of prev_snapshot
        if arr <> '' then
            part := part || ' union all '
                || select_fields || ' from ' || tbl || ' ev '
                || ' where ev.event_txid in (' || arr || ')'
                || retry_expr;
        end if;
        if sql = '' then
            sql := part;
        else
            sql := sql || ' union all ' || part;
        end if;
    end loop;
    if sql = '' then
        raise exception 'could not construct sql for batch %', x_batch_id;
    end if;
    return sql || ' order by 1';
end;
$$ language plpgsql;  -- no perms needed

