create or replace function pgq.maint_rotate_tables_step1()
returns integer as $$
-- rotates event tables
-- returns count of tables rotated
begin
    perform pgq.maint_rotate_tables_step1(queue_name)
        from pgq.queue_config;
    return 1;
end;
$$ language plpgsql; -- need admin access

create or replace function pgq.maint_rotate_tables_step1(i_queue_id integer)
returns integer as $$
-- temporary compat func
declare
    res integer;
begin
    res := pgq.maint_rotate_tables_step1(queue_name) from pgq.queue_config
        where queue_id = i_queue_id;
    return res;
end;
$$ language plpgsql; -- need admin access

create or replace function pgq.maint_rotate_tables_step1(i_queue_name text)
returns integer as $$
-- rotates event tables
declare
    badcnt  integer;
    cf      record;
    nr      integer;
    tbl     text;
begin
    -- check if needed and load record
    select * from pgq.queue_config
        into cf
        where queue_name = i_queue_name
          and rotation_delay is not null
          and last_switch_step2 is not null
          and last_switch_date + rotation_delay < current_timestamp
        for update;
    if not found then
        return 0;
    end if;

    -- check if any consumer is on previous table
    select coalesce(count(*), 0) into badcnt
        from pgq.high_watermark wm, pgq.tick t
        where get_snapshot_xmin(t.snapshot) < cf.last_switch_step2
          and wm.queue_id = cf.queue_id
          and t.queue_id = cf.queue_id
          and t.tick_id = (select tick_id from pgq.tick
                           where tick_id < wm.last_tick_id
                             and queue_id = wm.queue_id
                           order by queue_id desc, tick_id desc
                           limit 1);
    if badcnt > 0 then
        return 0;
    end if;

    -- check if any consumer has active batch on previous table
    select coalesce(count(*), 0) into badcnt
        from pgq.active_batch b, pgq.tick t
        where get_snapshot_xmin(t.snapshot) < cf.last_switch_step2
          and b.queue_id = cf.queue_id
          and t.tick_id = b.prev_tick_id
          and t.queue_id = cf.queue_id;
    if badcnt > 0 then
        return 0;
    end if;

    -- all is fine, calc next table number
    nr := cf.active_event_table + 1;
    if nr = cf.number_of_event_tables then
        nr := 0;
    end if;
    tbl := cf.table_name || '_' || nr;

    -- there may be long lock on the table from pg_dump,
    -- detect it and skip rotate then
    begin
        execute 'lock table ' || tbl || ' nowait';
        execute 'truncate ' || tbl;
    exception
        when lock_not_available then
            raise warning 'truncate of % failed, skipping rotate', tbl;
            return 0;
    end;

    -- remember the moment
    update pgq.queue_config
        set active_event_table = nr,
            last_switch_date = current_timestamp,
            last_switch_step1 = get_current_txid(),
            last_switch_step2 = NULL
        where queue_id = cf.queue_id;

    -- clean ticks - avoid partial batches
    delete from pgq.tick
        where queue_id = cf.queue_id
          and get_snapshot_xmin(snapshot) < cf.last_switch_step2;

    return 1;
end;
$$ language plpgsql; -- need admin access

create or replace function pgq.maint_rotate_tables_step2()
returns integer as $$
-- visibility tracking.  this should run in separate
-- tranaction than step1
begin
    update pgq.queue_config
        set last_switch_step2 = get_current_txid()
        where last_switch_step2 is null;
    return 1;
end;
$$ language plpgsql; -- need admin access

