
create or replace function pgq.next_batch(
    x_queue_name text,
    x_consumer_name text,
    x_thread_id text)
returns bigint as $$
--
-- get a new batch for consumer
--
declare
    last_tick      bigint;
    next_batch     bigint;
    next_tick      bigint;
    errmsg         text;
    cf             record;
    x_consumer_id  integer;
begin
    select queue_id into cf
        from pgq.queue_config
        where queue_name = x_queue_name;
    if not found then
        errmsg := 'No such event queue: '
            || coalesce(x_queue_name, 'NULL');
        raise exception '%', errmsg;
    end if;

    select consumer_id into x_consumer_id
        from pgq.consumer
        where consumer_name = x_consumer_name;
    if not found then
        errmsg := 'No such consumer: ' || coalesce(x_consumer_name, 'NULL');
        raise exception '%', errmsg;
    end if;

    -- lock the row
    select wm.last_tick_id into last_tick
        from pgq.high_watermark wm
        where wm.consumer_id = x_consumer_id
          and wm.queue_id = cf.queue_id
        for update;
    if not found then
        errmsg := 'Consumer '
            || coalesce(x_consumer_name, 'NULL')
            || ' is not registered for event '
            || coalesce(x_queue_name, 'NULL');
        raise exception '%', errmsg;
    end if;

    -- does the thread have pending batch?
    select batch_id into next_batch
        from pgq.active_batch
        where consumer_id = x_consumer_id
          and thread_id = x_thread_id
          and queue_id = cf.queue_id;
    if found then
        update pgq.active_batch
            set last_modified_date = current_timestamp
            where batch_id = next_batch;
        return next_batch;
    end if;

    -- find next tick
    select tick_id into next_tick
        from pgq.tick
        where tick_id > last_tick
          and queue_id = cf.queue_id
        order by queue_id asc, tick_id asc
        limit 1;
    if not found then
        -- nothing to do
        return null;
    end if;

    -- tag the tick as reserved
    insert into pgq.active_batch (queue_id,
            consumer_id, thread_id, tick_id, prev_tick_id)
        values (cf.queue_id,
                x_consumer_id, x_thread_id, next_tick, last_tick);
    next_batch := currval('pgq.active_batch_batch_id_seq');

    -- remeber last value
    update pgq.high_watermark
        set last_tick_id = next_tick, last_modified_date = current_timestamp
        where consumer_id = x_consumer_id
          and queue_id = cf.queue_id;

    return next_batch;
end;
$$ language plpgsql security definer;

