
SET client_min_messages = 'warning';
SET default_with_oids = false;

CREATE SCHEMA pgq;

-- Consumers
CREATE TABLE pgq.consumer  (
    consumer_id      serial      NOT NULL,
    consumer_name    text        NOT NULL,
    
    PRIMARY KEY (consumer_id)
);
CREATE UNIQUE INDEX consumer_name_idx ON pgq.consumer
    (consumer_name);

-- Queue information
CREATE TABLE pgq.queue_config  (
    queue_id               serial         NOT NULL,
    queue_name             text           NOT NULL,
    -- data tables
    table_name             text           NOT NULL,
    number_of_event_tables smallint       NOT NULL,
    active_event_table     smallint       NOT NULL,
    -- rotation
    rotation_delay         interval       NOT NULL DEFAULT '2 hours',
    last_switch_date       timestamptz    NOT NULL DEFAULT current_timestamp,
    last_switch_step1      bigint         NOT NULL DEFAULT get_current_txid(),
    last_switch_step2      bigint         NULL,
    -- event behaviour
    event_seq_name         text           NOT NULL,
    event_timeout          interval       NULL, -- unused
    -- batch log config
    use_batch_log          boolean        NOT NULL DEFAULT true,
    batch_log_timeout      interval       NOT NULL DEFAULT '10 minutes',
    -- ticker config
    external_ticker        boolean        NOT NULL DEFAULT false,
    tick_seq_name          text           NOT NULL,
    tick_idle_interval     integer        NOT NULL DEFAULT 60,
    tick_max_lag           integer        NOT NULL DEFAULT 2,
    tick_max_amount        integer        NOT NULL DEFAULT 200,
    -- time stamps
    creation_date          timestamptz    NOT NULL DEFAULT current_timestamp,
    last_modified_date     timestamptz    NOT NULL DEFAULT current_timestamp,

    PRIMARY KEY (queue_id)
);
CREATE UNIQUE INDEX queue_config_name_idx ON pgq.queue_config
    (queue_name);

-- Registered events for consumers and status
CREATE TABLE pgq.high_watermark  (
    consumer_id          integer          NOT NULL,
    queue_id             integer          NOT NULL,
    last_tick_id         bigint           NOT NULL,
    creation_date        timestamptz      NOT NULL DEFAULT current_timestamp,
    last_modified_date   timestamptz      NOT NULL DEFAULT current_timestamp,
    thread_timeout       interval,

    PRIMARY KEY (queue_id, consumer_id)
);

-- Active batches
CREATE TABLE pgq.active_batch  (
    batch_id             bigserial,
    queue_id             integer          NOT NULL,
    consumer_id          integer          NOT NULL,
    thread_id            text             NOT NULL,
    creation_date        timestamptz      NOT NULL DEFAULT current_timestamp,
    last_modified_date   timestamptz      NOT NULL DEFAULT current_timestamp,
    tick_id              bigint           NOT NULL,
    prev_tick_id         bigint           NOT NULL,

    PRIMARY KEY (batch_id)
);
CREATE UNIQUE INDEX active_batch_idx ON pgq.active_batch
    (queue_id, consumer_id, thread_id);

-- template table for event data
CREATE TABLE pgq.event_template (
    event_id           bigint      NOT NULL,
    event_txid         bigint      NOT NULL DEFAULT get_current_txid(),
    creation_date      timestamptz NOT NULL DEFAULT current_timestamp,
    ref_id             text,
    payload            text,
    producer           text,
    retry_count        integer,
    consumer_id        integer
);

-- table for snapshots
CREATE TABLE pgq.tick (
    queue_id         integer          NOT NULL,
    tick_id          bigint           NOT NULL,
    snapshot         txid_snapshot    NOT NULL DEFAULT get_current_snapshot(),
    tick_time        timestamptz      NOT NULL DEFAULT current_timestamp,

    PRIMARY KEY (queue_id, tick_id)
);

-- Statistics about every batch
CREATE TABLE pgq.batch_log (
    batch_id            bigint        NOT NULL,
    queue_id            integer       NOT NULL,
    consumer_id         integer       NOT NULL,
    thread_id           text          NOT NULL,  -- who started the batch
    tick_start          timestamptz   NOT NULL,
    tick_end            timestamptz   NOT NULL,
    batch_start         timestamptz   NOT NULL,
    batch_end           timestamptz,
    event_count         integer       NOT NULL default 0,
    retry_count         integer       NOT NULL default 0,
    failed_count        integer       NOT NULL default 0,

    PRIMARY KEY (batch_id)
);
CREATE INDEX batch_log_cons_idx ON pgq.batch_log
    (consumer_id, queue_id, batch_start);
CREATE INDEX batch_log_time_idx ON pgq.batch_log
    (queue_id, batch_start);

-- Retry events for consumer with data
CREATE TABLE pgq.retry_event  (
    consumer_id          integer          NOT NULL,
    queue_id             integer          NOT NULL,
    event_id             bigint           NOT NULL,
    retry_count          integer          NOT NULL,
    retry_after_date     timestamptz      NOT NULL,
    creation_date        timestamptz      NOT NULL,
    last_modified_date   timestamptz      NOT NULL DEFAULT current_timestamp,
    batch_id             bigint           NOT NULL,
    ref_id               text,
    payload              text,
    producer             text,

    PRIMARY KEY (queue_id, consumer_id, event_id)
);
CREATE INDEX retry_event_date_idx ON pgq.retry_event ( retry_after_date );
CREATE INDEX retry_event_batch_idx ON pgq.retry_event ( batch_id );

-- failed events for consumer with data
CREATE TABLE pgq.failed_event  (
    consumer_id          integer          NOT NULL,
    queue_id             integer          NOT NULL,
    event_id             bigint           NOT NULL,
    retry_count          integer          NOT NULL,
    creation_date        timestamptz      NOT NULL,
    failure_date         timestamptz      NOT NULL DEFAULT current_timestamp,
    failure_reason       text,
    batch_id             bigint,
    ref_id               text,
    payload              text,
    producer             text,

    PRIMARY KEY (queue_id, consumer_id, event_id)
);
CREATE INDEX failed_event_date_idx ON pgq.failed_event ( failure_date );
CREATE INDEX failed_event_batch_idx ON pgq.failed_event ( batch_id );

